% Example_bipartitePLMM_11Apr13.m
 
% Simulates models used to test the bipartite PLMM models

% requires code file bipartitePLMM.m with its dependencies
% requires data files:
% 	plantVforMatlab_14Apr11.txt
% 	insectVforMatlab_7Apr11.txt

reps=3;
model_1_flag=0;

annealingflag=1;
MLflag=1;
 
options=optimset('MaxFunEvals',5*10^3,'MaxIter',5*10^3,'TolX',10^-6,'TolFun',10^-6);
annealingopts=struct('StopTemp',10^-3,'Verbosity',0);

% load plant tree
plantV=dlmread('plantVforMatlab_14Apr11.txt');
plantV=plantV/max(plantV(:));

% load insect tree
insectV=dlmread('insectVforMatlab_7Apr11.txt');
insectV=insectV+insectV';
insectV=350-insectV;
insectV=insectV/max(insectV(:));

nspp=length(insectV(1,:));
mspp=length(plantV(1,:));

for caseflag=1:3

	if caseflag==1
		% variation in abundance among insects
		sb=1;
		iDins_a=zeros(nspp);

		% variation in response to trait among insects
		sslope=0;
		iDins=zeros(nspp);

		% variation in plant traits
		mplt=1;
		splt=1;
		iDplt=chol(splt*plantV)';

		se=.5;
	end
	
	if caseflag==2
		% variation in abundance among insects
		sb=0;
		sins_a=1;
		iDins_a=chol(sins_a*insectV)';

		% variation in response to trait among insects
		sslope=1;
		iDins=zeros(nspp);

		% variation in plant traits
		mplt=0;
		splt=1;
		iDplt=chol(splt*plantV)';

		se=.5;
	end

	if caseflag==3
		% variation in abundance among insects
		sb=0;
		sins_a=1;
		iDins_a=chol(sins_a*insectV)';

		% variation in response to trait among insects
		sslope=0;
		sins=1;
		iDins=chol(sins*insectV)';

		% variation in plant traits
		mplt=0;
		splt=1;
		iDplt=chol(splt*plantV)';

		se=.5;
	end

	soutlist=[];
	for rep=1:reps

		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		% simulate data
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

		b=sb*randn(nspp,1);
		a=iDins_a*randn(nspp,1);

		c=sslope*randn(nspp,1);
		d=iDins*randn(nspp,1);

		A=kron(ones(mspp,1),eye(nspp))*(a+b);
		B=mplt+kron(ones(mspp,1),eye(nspp))*(c+d);

		trt1=iDplt*randn(mspp,1);
		X1=kron(eye(mspp),ones(nspp,1))*trt1;

		simY=A+B.*X1+se*randn(mspp*nspp,1);

		if 0
			% print simY
			round(10*reshape(simY,mspp,nspp))
			sum(round(10*reshape(simY,mspp,nspp)))
			sum(round(10*reshape(simY,mspp,nspp))')'

			round(10*corrcoef(round(10*reshape(simY,mspp,nspp))))
			round(10*insectV)
			round(10*corrcoef(round(10*reshape(simY,mspp,nspp))')')
			round(10*plantV)
		end

		if model_1_flag
			%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
			% model 1:analyze trait data
			%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
			Vfullspp=kron(ones(mspp),eye(nspp));
			VfullsppV=kron(ones(mspp),insectV);

			Xspp=kron(ones(mspp,1),eye(nspp));
			XX=[X1 Xspp];

			u=trt1;
			u=(u-mean(u))/var(u)^.5;
			U=kron(u,ones(nspp,1));

			VfullUCU=diag(U)*Vfullspp*diag(U);
			VV=VfullUCU;

			VfullUCUV=diag(U)*VfullsppV*diag(U);
			VV(:,:,2)=VfullUCUV;

			sVV=size(VV);
			s2_init=1*ones(1,sVV(3));
			[BB,Bse,s2,s2e,lnlike,AIC,H,cH,exitflag]=bipartitePLMM(simY,XX,mspp,nspp,VV,s2_init,MLflag,annealingflag,options,annealingopts);
			B1=BB(1);
			B1se=Bse(1);
			
			REML_lnlike=lnlike;
			
			[rep s2e*[s2 1]  B1./B1se REML_lnlike exitflag]
			sout=[s2e*[s2 1] REML_lnlike exitflag];
			soutlist=[soutlist;sout];
	
			filename=['outputfile_Eq1_',num2str(caseflag),'.txt'];
			save(filename,'sout','-append','-ascii')
			
			figure(101)
			subplot(2,1,1)
			hist(H(:))
			xlabel('Residuals')
			ylabel('Frequency')
			
			subplot(2,1,2)
			hist(cH(:))
			xlabel('Cholesky Residuals')
			ylabel('Frequency')
			
			drawnow
		end

		if ~model_1_flag
			%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
			% model 2: estimate phylo signal
			%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

			XX=ones(mspp*nspp,1);

			Va=kron(ones(mspp),eye(nspp));
			Vb=kron(ones(mspp),insectV);
			Vc=kron(eye(mspp),insectV);

			Vd=kron(eye(mspp),ones(nspp));
			Vf=kron(plantV,ones(nspp));
			Vg=kron(plantV,eye(nspp));

			Vh=kron(plantV,insectV);

			VV=Va;
			VV(:,:,2)=Vb;
			VV(:,:,3)=Vc;
			VV(:,:,4)=Vd;
			VV(:,:,5)=Vf;
			VV(:,:,6)=Vg;
			VV(:,:,7)=Vh;

			sVV=size(VV);
			s2_init=.2*ones(1,sVV(3));

			[BB,Bse,s2,s2e,lnlike,AIC,H,cH,exitflag]=bipartitePLMM(simY,XX,mspp,nspp,VV,s2_init,MLflag,annealingflag,options,annealingopts);
			
			REML_lnlike=lnlike;

			[rep s2e*[s2 1] REML_lnlike exitflag]
			sout=[s2e*[s2 1] REML_lnlike exitflag];
			soutlist=[soutlist;sout];
	
			filename=['outputfile_Eq2_',num2str(caseflag),'.txt'];
			save(filename,'sout','-append','-ascii')

			figure(101)
			subplot(2,1,1)
			hist(H(:))
			xlabel('Residuals')
			ylabel('Frequency')
			
			subplot(2,1,2)
			hist(cH(:))
			xlabel('Cholesky Residuals')
			ylabel('Frequency')
		
			drawnow
		end
	end

	sout=soutlist(find(soutlist(:,end)==1),:);
	proportionConverged=length(sout(:,1))/reps

	figure(caseflag)
	set(caseflag,'position',[965   111   384   718])
	for i=1:sVV(3)
		ss=sort(sout(:,i));
		subplot(7,1,i)
		hist(ss,.05:.1:2.95)
		m=hist(ss,.05:.1:2.95);
		axis([0 3 0 max(m)+2])
	end
	drawnow
end